@@ -51,33 +51,43 @@ class Predictor(state.HasState):
      1             6.1            3               4.6            1.4         1  1.56469
      2             6.6            2.9             4.6            1.3         1  1.44276
     '''
-
+    snake_name = 'sklearn_predictor'
     model = traitlets.Any(default_value=None, allow_none=True, help='A scikit-learn estimator.').tag(**serialize_pickle)
     features = traitlets.List(traitlets.Unicode(), help='List of features to use.')
-    target = traitlets.Unicode(allow_none=False, help='The name of the target column.')
+    target = traitlets.Unicode(default_value=None, allow_none=True, help='The name of the target column.')
     prediction_name = traitlets.Unicode(default_value='prediction', help='The name of the virtual column housing the predictions.')
+    prediction_type = traitlets.Enum(values=['predict', 'predict_proba', 'predict_log_proba'], default_value='predict',
+                                     help='Which method to use to get the predictions. \
+                                     Can be "predict", "predict_proba" or "predict_log_proba".')
+
+    # if not hasattr(model, prediction_type):
+    #     raise AttributeError(f'The specified sklearn model does not have a {prediction_type} attribute')
 
     def __call__(self, *args):
-        X = np.vstack([arg.astype(np.float64) for arg in args]).T.copy()
-        return self.model.predict(X)
+        X = np.stack([np.asarray(arg, np.float64) for arg in args], axis=1)
+        if self.prediction_type == 'predict':
+            return self.model.predict(X)
+        elif self.prediction_type == 'predict_proba':
+            return self.model.predict_proba(X)
+        else:
+            return self.model.predict_log_proba(X)
 
     def predict(self, df):
-        '''Get an in-memory numpy array with the predictions of the SKLearnPredictor.self
+        '''Get an in-memory numpy array with the predictions of the Predictor.
 
         :param df: A vaex DataFrame, containing the input features.
-        :returns: A in-memory numpy array containing the SKLearnPredictor predictions.
+        :returns: A in-memory numpy array containing the Predictor predictions.
         :rtype: numpy.array
         '''
-        data = df[self.features].values
-        return self.model.predict(data)
+        return self.transform(df)[self.prediction_name].values
 
     def transform(self, df):
-        '''Transform a DataFrame such that it contains the predictions of the SKLearnPredictor.
+        '''Transform a DataFrame such that it contains the predictions of the Predictor.
         in form of a virtual column.
 
         :param df: A vaex DataFrame.
 
-        :return copy: A shallow copy of the DataFrame that includes the SKLearnPredictor prediction as a virtual column.
+        :return copy: A shallow copy of the DataFrame that includes the Predictor prediction as a virtual column.
         :rtype: DataFrame
         '''
         copy = df.copy()
@@ -87,27 +97,19 @@ class Predictor(state.HasState):
         return copy
 
     def fit(self, df, **kwargs):
-        '''Fit the SKLearnPredictor to the DataFrame.
+        '''Fit the Predictor to the DataFrame.
 
         :param df: A vaex DataFrame containing the features and target on which to train the model.
         '''
 
         X = df[self.features].values
-        y = df.evaluate(self.target)
+        if self.target is not None:
+            y = df.evaluate(self.target)
+        else:
+            y = None
         self.model.fit(X=X, y=y, **kwargs)
 
 
-@vaex.serialize.register
-@generate.register
-class SKLearnPredictor(Predictor):
-
-    def __init__(self):
-        super(SKLearnPredictor, self).__init__()
-        warnings.warn(message='''This class is deprecated and it will be removed in vaex-ml 0.8.
-                      Please use vaex.ml.sklearn.Predictor instead.''',
-                      category=DeprecationWarning)
-
-
 @vaex.serialize.register
 @generate.register
 class IncrementalPredictor(state.HasState):
@@ -173,17 +175,25 @@ class IncrementalPredictor(state.HasState):
     num_epochs = traitlets.Int(default_value=1, allow_none=False, help='Number of times each batch is sent to the model.')
     shuffle = traitlets.Bool(default_value=False, allow_none=False, help='If True, shuffle the samples before sending them to the model.')
     prediction_name = traitlets.Unicode(default_value='prediction', help='The name of the virtual column housing the predictions.')
+    prediction_type = traitlets.Enum(values=['predict', 'predict_proba', 'predict_log_proba'], default_value='predict',
+                                     help='Which method to use to get the predictions. \
+                                     Can be "predict", "predict_proba" or "predict_log_proba".')
     partial_fit_kwargs = traitlets.Dict(default_value={}, help='A dictionary of key word arguments to be passed on to the `fit_predict` method of the `model`.')
 
     def __call__(self, *args):
-        X = np.vstack([arg.astype(np.float64) for arg in args]).T.copy()
-        return self.model.predict(X)
+        X = np.stack([np.asarray(arg, np.float64) for arg in args], axis=1)
+        if self.prediction_type == 'predict':
+            return self.model.predict(X)
+        elif self.prediction_type == 'predict_proba':
+            return self.model.predict_proba(X)
+        else:
+            return self.model.predict_log_proba(X)
 
     def predict(self, df):
-        '''Get an in-memory numpy array with the predictions of the SKLearnPredictor.self
+        '''Get an in-memory numpy array with the predictions of the Predictor
 
         :param df: A vaex DataFrame, containing the input features.
-        :returns: A in-memory numpy array containing the SKLearnPredictor predictions.
+        :returns: A in-memory numpy array containing the Predictor predictions.
         :rtype: numpy.array
         '''
 
@@ -222,7 +232,7 @@ class IncrementalPredictor(state.HasState):
         expressions = self.features + [self.target]
 
         for epoch in range(self.num_epochs):
-            for i1, i2, chunks in df.evaluate_iterator(expressions, chunk_size=self.batch_size):
+            for i1, i2, chunks in df.evaluate_iterator(expressions, chunk_size=self.batch_size, array_type='numpy'):
                 progressbar((n_samples * epoch + i1) / (self.num_epochs * n_samples))
                 X = np.array(chunks[:-1]).T  # the most efficient way depends on the algorithm (row of column based access)
                 y = np.array(chunks[-1], copy=False)
